;=====Input_Str by Donald Straney=====
;
;188 bytes (including lookup table)
;
;Inputs: _penCol,_penRow-position of small cursor to start
;Outputs: (string stored in memory, zero-terminated, at (Str_Copy_Loc))


Input_Str:
    push hl                             ;Save position in memory
    ld hl,(_penCol)                     ;Look at cursor
    ld a,l
    cp 128-4                            ;Is it at the end of the line?
    call nc,_vnewline                   ;If so, move to the next line
    push hl                             ;Now save cursor position
    ld hl,Input_Str_Cursor_Str          ;Draw cursor on screen
    call _vputs
    pop hl                              ;Restore cursor position-just to left of graphical cursor
    ld (_penCol),hl
    call _getkey                        ;Wait for keypress
    pop hl                              ;(restore position in memory)
    cp kExit                            ;Has the exit key been pressed?
    jp z,_jforcecmdnochar               ;If so, go back to the homescreen
    cp kEnter                           ;Is it the enter key?
    jr z,Input_Str_End                    ;If so, terminate string and end
    cp kDel                             ;Is it the delete key?
    jr z,Input_Str_Del                    ;If so, delete last char
    push hl
    ld de,Str_Copy_Loc+Str_Max_Len      ;Now, are we at the max number of chars allowed?
    sbc hl,de
    pop hl
    jr z,Input_Str                        ;If we are, go back and don't allow any keypresses except enter, exit, and delete
    cp ka                               ;If it's a lowercase letter, display that
    jr nc,Input_Str_L_Letter
Input_Str_Test_More_Keys:
    cp kCapA                            ;If it's an uppercase letter, display that
    jr nc,Input_Str_U_Letter
Input_Str_Test_More_Keys_2:
    cp k0                               ;If it's a number, display that
    jr nc,Input_Str_Number
Input_Str_Find_In_Table:                  ;Otherwise, see if this key is supported
    ld b,a                              ;Save keycode
    ld de,Input_Str_Table_Start
Input_Str_Find_In_Table_Loop:
    ld a,(de)                           ;Check keycode against table
    or a                                ;(is it the end of the table?)
    jr z,Input_Str                        ;(if so, it's not in the table-go back and wait for another keypress)
    cp b                                ;Is it supported?
    jr z,Input_Str_Find_In_Table_End      ;If it is, get the equivalent char code
    inc de                              ;If it isn't, look at the next entry
    inc de
    jr Input_Str_Find_In_Table_Loop       ;and loop
Input_Str_Find_In_Table_End:
    inc de                              ;Equivalent char code is stored right after keycode, so point to that
    ld a,(de)                           ;Get charcode in a
Input_Str_Insert_Char:
    ld (hl),a                           ;Store it in memory
    inc hl                              ;Point to next memory location
    ld de,(_penCol)                     ;Save cursor position for this char on stack
    push de
    call _vputmap                       ;and draw char on screen
    jr Input_Str                          ;Go back and wait for next keypress
Input_Str_U_Letter:
    cp kCapZ+1                          ;Is it too high too be an uppercase letter?
    jr nc,Input_Str_Test_More_Keys_2      ;If so, go back
    add a,$41-$28                       ;Otherwise, convert from keycode to charcode
    jr Input_Str_Insert_Char
Input_Str_L_Letter:
    cp kz+1                             ;Is it too high to be a lowercase letter?
    jr nc,Input_Str_Test_More_Keys        ;If so, go back
    add a,$61-$42                       ;Otherwise, convert from keycode to charcode
    jr Input_Str_Insert_Char
Input_Str_Number:
    cp k9+1                             ;Is it too high to be a number?
    jr nc,Input_Str_Find_In_Table         ;If so, go back
    add a,$30-$1c                       ;Otherwise, convert from keycode to charcode
    jr Input_Str_Insert_Char
Input_Str_End:
    ld (hl),0                           ;Zero-terminate string in memory
    ld de,Str_Copy_Loc                  ;Get the number of chars in the string
Input_Str_Clean_Stack:
    sbc hl,de
    ret z                               ;If none, then don't need to take any positions off stack-return
    ld b,l
Input_Str_End_Loop:
    pop de                              ;Using that, remove the cursor positions of
    djnz Input_Str_End_Loop               ;each char that were stored on the stack
    ret
Input_Str_Del:
    push hl
    ld de,Str_Copy_Loc                  ;Is user trying to backspace too far?
    sbc hl,de
    pop hl
    jp z,Input_Str                        ;If so, don't allow it-wait for another keypress
    dec hl                              ;Point to last char
    ld (hl),0                           ;Reset it
    push hl
    ld b,4                              ;Draw 4 spaces to erase graphical cursor
Input_Str_Del_Loop:
    ld a,Lspace
    push bc
    call _vputmap
    pop bc
    djnz Input_Str_Del_Loop
    pop hl
    pop de                              ;Now get cursor position of last char
    ld (_penCol),de                     ;Set cursor to that, so that cursor will have appeared to move backwards
    jp Input_Str                          ;Now wait for another keypress

Input_Str_Table_Start:                  ;Lookup table of keycode-to-charcode
.db kLParen,LlParen
.db kRParen,LrParen
.db kDecPnt,Lperiod
.db kSqrt,Lroot
.db kAdd,LplusSign
.db kSub,Ldash
.db kMul,Lasterisk
.db kDiv,Lslash
.db kExpon,Lcaret
.db 0

_vnewline:                              ;Cursor position is stored in hl for now, so don't need to change the actual cursor position in memory
    ld a,h                              ;Move cursor down 1 row
    add a,6
    ld h,a
    ld l,0                              ;and to the beginning of the next line
    ret

Input_Str_Cursor_Str: .db Lunderscore,Lspace,0

;=====end of routine=====

.end